/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/
package org.jboss.classloading.plugins;

import java.net.URL;
import java.util.Set;

import org.jboss.classloading.spi.ClassLoadingDomain;
import org.jboss.classloading.spi.DomainClassLoader;
import org.jboss.util.ClassLoading;
import org.jboss.util.JBossObject;
import org.jboss.util.JBossStringBuilder;

/**
 * A delegate domain classloader.
 * 
 * @author <a href="adrian@jboss.com">Adrian Brock</a>
 * @version $Revision: 1.4 $
 */
public class DelegatingDomainClassLoader extends JBossObject implements DomainClassLoader
{
   // Constants -----------------------------------------------------

   // Attributes ----------------------------------------------------

   /** The domain */
   protected ClassLoadingDomain domain;
   
   /** The delegate classloader */
   protected WrappingClassLoader delegate;
   
   // Static --------------------------------------------------------

   // Constructors --------------------------------------------------
   
   // Public --------------------------------------------------------
   
   /**
    * Create a new Delegating domain classloader
    * 
    * @param delegate the delegate classloader
    */
   public DelegatingDomainClassLoader(ClassLoader delegate)
   {
      this.delegate = new WrappingClassLoader(delegate);
   }
   
   // DomainClassLoader implementation ------------------------------

   public ClassLoadingDomain getDomain()
   {
      return domain;
   }

   public void setDomain(ClassLoadingDomain domain)
   {
      this.domain = domain;
   }

   public Class loadClassLocally(String name, boolean resolve) throws ClassNotFoundException
   {
      return delegate.getParent().loadClass(name);
   }
   
   public URL loadResourceLocally(String name)
   {
      return delegate.getParent().getResource(name);
   }

   public Set getPackageNames()
   {
      return ClassLoading.getClassLoaderSpecificPackages(delegate);
   }
   
   public Package getPackage(String name)
   {
      return delegate.getPackage(name);
   }

   // JBossObject overrides -----------------------------------------
   
   public void toString(JBossStringBuilder buffer)
   {
      buffer.append("classLoader=" + delegate.getParent());
   }

   public void toShortString(JBossStringBuilder buffer)
   {
      buffer.append(delegate.getParent());
   }
   
   // Package protected ---------------------------------------------

   // Protected -----------------------------------------------------
   
   // Private -------------------------------------------------------

   // Inner classes -------------------------------------------------
}