/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/
package org.jboss.kernel.plugins.deployment;

import java.util.List;

import org.jboss.beans.metadata.spi.ClassLoaderMetaData;
import org.jboss.kernel.spi.dependency.KernelControllerContext;
import org.jboss.kernel.spi.deployment.KernelDeployment;
import org.jboss.util.CollectionsFactory;
import org.jboss.util.JBossObject;
import org.jboss.util.JBossStringBuilder;

/**
 * An abstract kernel deployment.
 * 
 * @author <a href="adrian@jboss.com">Adrian Brock</a>
 * @version $Revision: 1.6 $
 */
public class AbstractKernelDeployment extends JBossObject implements KernelDeployment
{
   /** The name of the deployment */
   protected String name;

   /** Whether it is installed */
   protected boolean installed;

   /** The installed contexts */
   protected List installedContexts = CollectionsFactory.createCopyOnWriteList();

   /** The beans List<BeanMetaData> */
   protected List beans;
   
   /** The ClassLoader */
   protected ClassLoaderMetaData classLoader;

   /**
    * Create a new kernel deployment
    */
   public AbstractKernelDeployment()
   {
   }

   /**
    * Set the beans.
    * 
    * @param beans a List<BeanMetaData>.
    */
   public void setBeans(List beans)
   {
      this.beans = beans;
      flushJBossObjectCache();
   }

   public String getName()
   {
      return name;
   }

   public void setName(String name)
   {
      this.name = name;
      flushJBossObjectCache();
   }
   
   public boolean isInstalled()
   {
      return installed;
   }

   public void setInstalled(boolean installed)
   {
      this.installed = installed;
      flushJBossObjectCache();
   }

   public void addInstalledContext(KernelControllerContext context)
   {
      installedContexts.add(context);
      flushJBossObjectCache();
   }

   public List getInstalledContexts()
   {
      return installedContexts;
   }

   public void removeInstalledContext(KernelControllerContext context)
   {
      installedContexts.remove(context);
      flushJBossObjectCache();
   }

   public List getBeans()
   {
      return beans;
   }

   public ClassLoaderMetaData getClassLoader()
   {
      return classLoader;
   }
   
   /**
    * Set the classloader
    * 
    * @param classLoader the classloader metadata
    */
   public void setClassLoader(ClassLoaderMetaData classLoader)
   {
      this.classLoader = classLoader;
   }
   
   public void toString(JBossStringBuilder buffer)
   {
      buffer.append("name=").append(name);
      buffer.append(" installed=").append(installed);
      if (classLoader != null)
         buffer.append(" classLoader=").append(classLoader);
      if (beans != null)
         buffer.append(" beans=").append(beans);
   }
   
   public void toShortString(JBossStringBuilder buffer)
   {
      buffer.append(name);
   }
}