/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/
package org.jboss.test.classloading.support;

import java.net.URL;
import java.util.HashMap;

/**
 * A simple classloader
 * 
 * @author <a href="adrian@jboss.com">Adrian Brock</a>
 * @version $Revision: 1.2 $
 */
public class SimpleClassLoader extends ClassLoader
{
   // Constants -----------------------------------------------------

   // Attributes ----------------------------------------------------

   /** The classes */
   public HashMap entries = new HashMap();
   
   /** The load class requests */
   public HashMap requests = new HashMap();
   
   // Static --------------------------------------------------------

   // Constructors --------------------------------------------------
   
   public SimpleClassLoader()
   {
      super(null);
   }
   
   // Public --------------------------------------------------------
   
   public void addClass(String className, Class clazz)
   {
      int dot = className.lastIndexOf('.');
      String packageName = className.substring(0, dot);
      definePackage(packageName, null, null, null, null, null, null, null);
      entries.put(className, clazz);
   }
   
   public void addResource(String name, URL url)
   {
      int dot = name.lastIndexOf('/');
      String packageName = name.substring(0, dot);
      definePackage(packageName.replace('/', '.'), null, null, null, null, null, null, null);
      entries.put(name, url);
   }
   
   // ClassLoader overrides -----------------------------------------

   public Class loadClass(String name) throws ClassNotFoundException
   {
      ClassLoaderStats lcr = (ClassLoaderStats) requests.get(name);
      if (lcr == null)
      {
         lcr = new ClassLoaderStats();
         requests.put(name, lcr);
      }
      lcr.loadClass++;
      Class clazz = (Class) entries.get(name);
      if (clazz == null)
         throw new ClassNotFoundException(name);
      else
         return clazz;
   }
   
   // Package protected ---------------------------------------------

   // Protected -----------------------------------------------------

   // Private -------------------------------------------------------

   // Inner classes -------------------------------------------------

   public URL getResource(String name)
   {
      ClassLoaderStats lcr = (ClassLoaderStats) requests.get(name);
      if (lcr == null)
      {
         lcr = new ClassLoaderStats();
         requests.put(name, lcr);
      }
      lcr.getResource++;
      return (URL) entries.get(name);
   }
}